local love = assert( love )
local lg = assert( love.graphics )
local scene = assert( require 'scene' )
local menu = {}

--Static variables.
local selectedButtonIdx
local canvas
local wWidth, wHeight
local currentMenu

function menu.getSelectedButton()
  return selectedButtonIdx and currentMenu.buttons[selectedButtonIdx]
end

function menu.selectButton( idx )
  local but = currentMenu.buttons[selectedButtonIdx]
  if but then but.selected = false end
  selectedButtonIdx = idx
  but = currentMenu.buttons[idx]
  if but then but.selected = true end
end

function menu.clear()
  return lg.clear( canvas )
end

function menu.new( t )
  if t.subScene then setmetatable( t, t )
  else
    scene[t.name] = t
  end
  getmetatable( t ).__index = menu
  return t
end

function menu:onLoad()
  print( 'Loading Menu:', self.name )
  currentMenu = self
  if self.font then lg.setFont( self.font ) end
  return menu.resize( lg.getDimensions() )
end

function menu.update( dt ) end

function menu.resize( w, h )
  wWidth, wHeight = w, h
  canvas = lg.newCanvas()
  return currentMenu:paint()
end

function menu.draw()
  lg.setCanvas()
  lg.setColor( 1,1,1,1 )
  lg.draw( canvas )
end

function menu.mousemoved( x, y, dx, dy, istouch )
  if not currentMenu then return end
  local buttons = currentMenu.buttons

  local selectedButton = buttons[selectedButtonIdx or 0]
  for id, menuButton in ipairs( buttons ) do
    if menuButton:contains( x, y ) then
      if selectedButton then selectedButton.selected = false end
      menuButton.selected = true
      selectedButtonIdx = id
      if menuButton ~= selectedButton then
        return currentMenu:paint()
      else return end
    end
  end

  --deselect button
  if selectedButton then
    selectedButtonIdx = nil
    selectedButton.selected = false
    return currentMenu:paint()
  end
end


function menu.mousepressed( x, y, button, istouch, presses )

  if not selectedButtonIdx and currentMenu then return end
  local uiButton = currentMenu.buttons[selectedButtonIdx]
  if uiButton:contains( x, y ) then
    selectedButtonIdx = nil
    uiButton.selected = false
    return uiButton:callback()
  end
  return currentMenu:paint()
end

function menu.keypressed( key, code, isrepeat )

  assert( currentMenu )

  if code == "escape" then
    return love.event.quit()
  end

  local buttons = currentMenu.buttons

  if code == "return" and selectedButtonIdx then
    local button = buttons[selectedButtonIdx]
    selectedButtonIdx = nil
    button.selected = false
    currentMenu:paint()
    return button:callback()
  end

  if #buttons > 0 and (code == "down" or code == "tab" or code == "up") then
    repeat
      local sbi = (selectedButtonIdx or 1)
      if buttons[sbi] then buttons[sbi].selected = false end

      --Increment / decrement

      sbi = sbi + ((code == "up") and -1 or 1)
      if #buttons < 1 then selectedButtonIdx = false; return end
      if sbi > #buttons then sbi = 1 end
      if sbi < 1 then sbi = #buttons end

      selectedButtonIdx = sbi
      buttons[selectedButtonIdx].selected = true
    until buttons[selectedButtonIdx].active --Skip deactivated buttons.
    print( "Selected button: ", selectedButtonIdx )
  end


  return currentMenu:paint()
end

function menu:paint()
  lg.setCanvas( canvas )

  --bg
  lg.setColor( 1, 1, 1, 1 )
  if self.bg then lg.draw( self.bg, 0, 0, 0, wWidth, wHeight ) end

  --buttons
  for i = #self.buttons, 1, -1 do self.buttons[i]:draw( ) end

  --gradient
  lg.setColor( 1, 1, 1, 1 )

  if self.fg then lg.draw( self.fg, 0, 0, 0, wWidth, wHeight ) end
  lg.setCanvas()
end


return menu