package.cpath = "..\\?.dll;..\\clib\\?.dll;"..package.cpath
package.path = "..\\?.lua;..\\lualib\\?.lua;"..package.path
local shared = assert( require 'shared.shared' )
local packet = shared.packet
local socket = assert( require 'socket' )
local mscxn = assert( socket.udp() )
mscxn:settimeout( 0 )
assert( mscxn:setpeername( shared.metaserver.ip, shared.metaserver.port ), "Could not connect to metaserver!" )
local udp
local io = assert( io )

local CLIENTTIMEOUT = 10

local svInfo = packet.serverInfo{ version = 13,
  players = 0,
  capacity = 255,
  ip = shared.ip.fromString( socket.dns.toip(socket.dns.gethostname()) ),
  port = 51312,
  svname = "New Server",
  map = "Test Map"
}

local clients = {}
local connecting = {}
local server = { tick = 0, time = 0, currentClient = false }

local handlers = setmetatable({

    --This is the start of a packet from a connected client.
    connected = function( msg, ip, port )
      local client = clients[msg.token]
      if not client or
      (client.ip ~= ip) or
      (client.port ~= port)
      then
        print( "Invalid token from IP:", msg.token, ip, port )
        return true
      end

      if client.tick > msg.tick then
        print( "Old packet received:", msg.tick, ip )
        return true
      end

      client.tick = msg.tick
      client.time = server.time
      server.currentClient = client
    end,

    --A client wants to disconnect.
    disconnect = function( msg, ip, port )
      --This is the authenticated client whose packets are being processed.
      local client = server.currentClient
      if client and client.ip == ip and client.port == port then
        print( "Client disconnecting:", client.id)
        clients[ client.id ] = nil
        server.currentClient = false
        return true
      end
    end,

    --Client responds to handshake challenge.
    clChimo = function( clChimo, ip, port )

      print( "Received handshake response." )

      --No active challenge, don't send anything.
      local key = ip..":"..port
      if not connecting[ key ] then
        print( "Old connection attempt from:", key )
        return true
      end

      --Compute session token.
      local remoteHash = clChimo.hash
      local clNonce = clChimo.nonce
      local svNonce = connecting[key].nonce
      local token = shared.hash.hash( clNonce, svNonce )
      if token ~= remoteHash then
        print( "Hashes differ:", shared.hash.hex( clNonce ), shared.hash.hex( svNonce ) )
        return true
      end

      --Hash collision.
      while clients[token] do token = token + 1 end

      --Successful handshake.
      print( "Client connected:", token, port, ip )
      clients[ token ] = connecting[ key ]
      clients[ token ].id = token
      clients[ token ].time = socket.gettime()
      packet.connected{ token = token, tick = server.tick }
      return udp:sendto( packet.get(), ip, port )
    end,

    --Metaserver replying with real IP address.
    advertised = function( ack, ip, port )
      if ip ~= shared.metaserver.ip then return print( "Advertisement acked from rogue address:", ip, port ) end
      if udp:getsockname() then 
        --assert( udp:getsockname() == tostring( ack.ip ), print( tostring( ack.ip ), udp:getsockname()) or "IP mismatch!" )
        return print( "Advertised. Address already set." )
      end
      print( "Advertised. Setting address:", ack.ip, ack.port )
      server.SetIP( tostring( ack.ip ), ack.port )
    end,

    --New client seeks to connect. Challenge immediately.
    clientInfo = function( clientInfo, ip, port )
      local key = ip..":"..port 
      connecting[key] = connecting[key] or { ip = ip, port = port, tick = 0 }
      local client = connecting[key]
      local nonce = shared.hash.rand()
      client.nonce = nonce
      print( "Received connection request from:", ip, port )
      print( "Sending authentication nonce:", nonce )
      packet.svChimo{ nonce = nonce }
      return udp:sendto( packet.get(), ip, port )
    end,


    }, {__index = function() return print end })

function server.Advertise()
  print( "Advertise." )
  packet.get()
  packet.metaServer()
  packet.serverInfo( svInfo )
  mscxn:send( packet.get() )
end

function server.serverInfo( serverInfo )
  print( packet.getString( serverInfo.svname ) )
end

function server.metaServer( ms )
  server.advertising = true
end

--Incoming packet.
function server.Parse( msg, ip, port )
  if (not msg) or (#msg < 1) then return end
  local msgs, types = packet.deserialise( msg )
  if msgs then
    for i = 1, #msgs do
      if handlers[ types[i] ]( msgs[i], ip, port ) then break end
    end
    server.currentClient = false
  else print( types )
  end
  return server.Parse( udp:receivefrom() ) -- Process other packets.
end

function server.SetIP( ipString, port )
  svInfo.ip = shared.ip.fromString( ipString )
  svInfo.port = port
  local ok, err = udp:setsockname( ipString, port )
  if ok then
    print( "Server IP:", udp:getsockname() )
    return true
    --Find another port.
  elseif port < 65536 then
    print( "Could not use port:", ipString, port, err )
    return server.SetIP( ipString, port + 1 )
  else
    print( "Could not use IP:", ipString, err )
    return error( "Connection failed." )
  end
end


function server.Start()
  udp = assert( socket.udp() )
  udp:settimeout(0)
end

function server.Advance()
  server.tick = server.tick + 1
  for id, client in pairs( clients ) do
    packet.get()
    packet.connected{ token = id, tick = server.tick }
    udp:sendto( packet.get(), client.ip, client.port )

    if server.time - client.time > CLIENTTIMEOUT then
      print( "dropping client:", id )
      clients[id] = nil
    end
  end
end

function server.NewGame()
  server.tick = 0
end

function server.Quit()

end

server.Start()
function love.update( dt )
  server.time = server.time + dt
  server.Parse( udp:receivefrom() )
  if server.time > 0.1 then
    if (server.tick % 50) == 0 then
      server.Advertise()
      server.Parse( mscxn:receive(), shared.metaserver.ip, shared.metaserver.port )
    end
    server.time = 0
    server.Advance()
  end
end

function love.draw() end